/* tool.c -- Magic C++ utilities programme
   (Mostly) portable public-domain implementation
   -- Copyright(C) 2003 Magicunix Infomation Technology Limited 
   This file is part of magicd.
   magicd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
	 
   For details, see the Magic C++ World-Wide-Web page, 
   `http://www.magicunix.com',
   or send a mail to the Magic C++ developers <support@magicunix.com>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>

#ifndef MINGW
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>
#endif

#include <time.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <string.h>
#include <unistd.h>
#include "config.h"
#include "tools.h"

extern int debug;   /*from magicd.c*/
extern char szInstallDir[ 200 ]; /*from info.h*/

#ifdef SOLARIS
 struct tm *localtime_r(const time_t *clock, struct tm *res);
#endif

#if /*defined(__GNU_LIBRARY__) &&*/ !defined(_SEM_SEMUN_UNDEFINED)
       /* union semun is defined by including <sys/sem.h> */
#else
       /* according to X/OPEN we have to define it ourselves */
       union semun {
             int val;                  /* value for SETVAL */
             struct semid_ds *buf;     /* buffer for IPC_STAT, IPC_SET */
             unsigned short *array;    /* array for GETALL, SETALL */
                                       /* Linux specific part: */
             struct seminfo *__buf;    /* buffer for IPC_INFO */
       };

#endif


       
int util_err_log(char * info ,char *  filename , int line , long err)
{
	return util_log( "Programme error: %s :filename:%s , line :%d , error:%s",
		info,filename , line , strerror( err ) );
}

/*output log infomation to log file
usage: util_log("%s%s",......);
output [time] ...
*/

int util_log(  const char *fmt, ...)
{
#ifndef MINGW
	FILE *lfd;
	char	lname[200];
	time_t	dt;
	struct	tm dc;
	va_list args;
	char buf[102400];
	int i;
	int max_fd;
	char *envhome;
	memset( buf , 0 , sizeof( buf ));
	va_start(args, fmt);
	i=vsprintf(buf,fmt,args);
	va_end(args);
	/*get time*/

	memset(lname,0,sizeof(lname));
	time(&dt);
	
	localtime_r(&dt,&dc);
	
	/*not initialize installation path yet*/
	if( szInstallDir[ 0 ] != '/' )
	{
		fprintf(stderr,"[%04d%02d%02d %02d:%02d:%02d]|%s|%d|%s\n",dc.tm_year+1900,dc.tm_mon+1,dc.tm_mday,dc.tm_hour,dc.tm_min,dc.tm_sec, __FILE__ ,__LINE__,strerror( errno ));
		return -1;
	}
	sprintf( lname , "%s/log/dup.log"  , szInstallDir);
	
	if ( (lfd = fopen(lname,"a+")) == NULL )
	{
		lfd = stderr;
		#ifndef MINGW
		max_fd = sysconf(_SC_OPEN_MAX);
		#endif
		for(i=3;i<max_fd;i++)
			close(i);
		fprintf(lfd,"[%04d%02d%02d %02d:%02d:%02d]|%s|%d|%s|%s\n",dc.tm_year+1900,dc.tm_mon+1,dc.tm_mday,dc.tm_hour,dc.tm_min,dc.tm_sec, __FILE__ ,__LINE__,buf,strerror( errno ));
		return -1;
		
	}

	fprintf(lfd,"[%04d%02d%02d %02d:%02d:%02d]%s\n",dc.tm_year+1900,dc.tm_mon+1,dc.tm_mday,dc.tm_hour,dc.tm_min,dc.tm_sec,buf); 
	fflush(lfd);
	fclose(lfd);
#endif	
	return 1;
}


/*initialize semaphore and shared memory*/
/*
int util_ipc_init()
{
	if( util_sem_init() == -1 )
	{
		util_log("Init semphone failed!\n");
		return -1;
	}
	if( util_shm_init() == -1 )
	{
		util_log("Init share memory failed!\n");
		return -1;
	}
	return 0;

}
key_t	shmkey;
key_t   semkey;
*/
/*
int util_sem_init()
{
	char lname[200];
	FILE *lfd;
	char *envhome;
	int semid;
	u_short	init[16];
	semid = util_getsem( IPC_CREAT|S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP| S_IROTH|S_IWOTH );
	if( semid == -1 )
		return -1;
	init[0]=1;
	
	if( szInstallDir[ 0 ] != '/' )
	{
		fprintf(stderr,"Can not get install dir\n");
		return -1;
	}

	sprintf( lname , "%s/log/sem.log" , szInstallDir);
	lfd = fopen( lname , "w+" );
	if( lfd == NULL )
	{
		util_err_log("Can not open sem log file!\n",__FILE__,__LINE__,errno );
		
	}
	fprintf( lfd , "%d\n", semid );
	fclose( lfd );

	if ( semctl(semid, 1, SETALL, init) == -1 )
	{
		util_err_log("semctl() error!",__FILE__,__LINE__,errno);
		return -1;
	}
	util_log("Init Semaphore finished...\n");
	return 0;	
}
*/
/*
int util_shm_init()
{
	char lname[200];
	int shmid;
	FILE *lfd;
	int *global;
	char * envhome;
	shmid = util_getshm( IPC_CREAT|S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP| S_IROTH|S_IWOTH  );
	if( shmid == -1 )
		return -1;

	if( szInstallDir[ 0 ] != '/' )
	{
		fprintf(stderr,"Can not get install dir\n");
		return -1;
	}
	sprintf( lname , "%s/log/shm.log" , szInstallDir );

	lfd = fopen( lname , "w+" );
	if( lfd == NULL )
	{
		util_err_log("Can not open sem log file!\n",__FILE__,__LINE__,errno );
		
	}
	fprintf( lfd , "%d\n", shmid );
	fclose( lfd );
	if( ( global = (int *)shmat(shmid , 0 , 0 ) ) == (void *)-1 )
	{
		util_err_log("shmat() error!",__FILE__,__LINE__,errno);
		return -1;
		
	}
	*global = 0;
	shmdt( (char *)global );
	util_log("Init global finished...\n");
	return 0;
}
*/
/*lock the shared memory*/
/*nMillionSec -- timeout (usecond)*/
/*
int util_sem_lock(int nMillionSec)
{

	int	semid;
	struct	sembuf lock[1];
	int times;
	semid = util_getsem( S_IRUSR|S_IWUSR );
	if( semid == -1 )
		return -1;
	lock[0].sem_num=0;
	lock[0].sem_op=-1;
	lock[0].sem_flg=IPC_NOWAIT;
	times = 0;
	

	while( times < RETRY_TIMES )
	{
		if ( semop(semid, lock, 1) == -1 )
		{
			
			if( errno == EAGAIN )
			{
				
				usleep( nMillionSec * 1000 / RETRY_TIMES  );
				times ++;
				continue;
			}
			else
			{
				util_err_log("semop error!" , __FILE__,__LINE__,errno);
				return -1;
			}
		}
		else
		{
			break;
		}
	}
	if( times == RETRY_TIMES )
	{
		return -1;
	}
	return 0;
}
*/
/*unlock semaphore*/
/*
int util_sem_unlock()
{

	int	semid;
	int z;
		
	union semun un;
	semid = util_getsem( S_IRUSR|S_IWUSR );
	if( semid == -1 )
		return -1;
	un.val = 1;
	z = semctl(semid, 0, SETVAL, un)  ;

	if( z == -1 )
	{
		util_err_log("semctl getval error!", __FILE__,__LINE__,errno);
		return -1;
	}
	return 0;
}

*/
/*add shared memory by 1*/
/*if it is up to limitationreturn -1 */
/*
int util_add_global(int maxval)
{
	int *global;
	int shmid;
	shmid = util_getshm(S_IRUSR|S_IWUSR);
	if( shmid == -1 )
		return -1;
	if( ( global = (int *)shmat(shmid , 0 , 0 ) ) == (void *)-1 )
	{
		util_err_log("shmat() error!",__FILE__,__LINE__,errno);
		return 0;
		
	}
	if( util_sem_lock(1000) == -1 )
	{
		util_err_log( "Can not get lock!",__FILE__,__LINE__,errno );
		return -1;
		
	}
	if( *global >= maxval )
	{
		
		memcpy( global , &maxval , sizeof( int ));
		util_log("The Licence count is up to limit!Can not apply any more...\n");
		util_sem_unlock();
		return -1;
	}
	else
	{
		int ntmp;
		ntmp = *global + 1;
		memcpy( global , &ntmp , sizeof( int ));

		util_log("Apply a Licence,Current Licence count is %d Now...\n",*global);

	}
	shmdt((char *) global );
	util_sem_unlock();
	return 0;	
	
}
*/   
/*subtract value in shared memory if it's 0return -1 */
/*
int util_subtract_global()
{
	int *global;
	int shmid;
	shmid = util_getshm( S_IRUSR|S_IWUSR );
	if( shmid == -1 )
		return -1;
	if( ( global = (int *)shmat(shmid , 0 , 0 ) ) == (void *)-1 )
	{
		util_err_log("shmat() error!",__FILE__,__LINE__,errno);
		return 0;
		
	}	
	if( util_sem_lock(1000) == -1 )
	{
		util_err_log( "Can not get lock!",__FILE__,__LINE__,errno );
		return -1;
		
	}
	if( *global <= 0 )
	{
		int ntmp = 0;
		memcpy( global , &ntmp , sizeof( int ));
		util_log("The Licence Count is zero Now...Can not release any more...\n");
		util_sem_unlock();
		return -1;
	}
	else
	{
		int ntmp = *global - 1;
		memcpy( global , &ntmp , sizeof( int ));

		util_log("Release a licence...Current Licence count is %d Now...\n",*global);		

	}
	shmdt((char *) global );
	util_sem_unlock();
	return 0;		
}


int util_getshm(int flag)
{
	 

	char	keyfile[100];
	int	shmid;
	int *global;
	
	memset(keyfile,0,sizeof(keyfile));
	if( szInstallDir[ 0 ] != '/' )
	{
		fprintf(stderr,"Can not get install dir\n");
		return -1;
	}
	sprintf( keyfile , "%s/etc/global.etc" , szInstallDir);
	shmkey = ftok( keyfile,'Y' );
	if ( shmkey == -1 )
	{
		util_err_log("ftok() error!" ,__FILE__,__LINE__,errno);
		return -1;
	}
	if ( ( shmid = shmget(shmkey, sizeof(int) , flag) ) < 0 )
	{
		util_err_log("shmget() error!" ,__FILE__,__LINE__,errno);
		return -1;
	}
	return shmid;
}

int util_getsem(int flag)
{ 	

	char	keyfile[100];
	int	semid;
	u_short	init[16];
	memset(keyfile,0,sizeof(keyfile));
	if( szInstallDir[ 0 ] != '/' )
	{
		fprintf(stderr,"Can not get install dir\n");
		return -1;
	}

	sprintf( keyfile , "%s/etc/sems.etc" , szInstallDir );
	semkey = ftok( keyfile,'Y' );
	if ( semkey == -1 )
	{
		util_err_log("ftok() error!" ,__FILE__,__LINE__,errno);
		return -1;
	}
	if ( ( semid = semget(semkey, 1 , flag) ) < 0 )
	{
		util_err_log("semget() error!" ,__FILE__,__LINE__,errno);
		return -1;
	}
	return semid;
	
}
*/
/*remove semaphore*/
/*
int util_ipc_remove()
{
	if( util_sem_remove() == -1 )
	{
		util_log("remove semphone failed!\n");
		return -1;
	}
	
	if( util_shm_remove() ==  -1 )
	{
		util_log("Remove Share memmory failed!\n");
		return -1;
		
	}
	return 0;
	
	

}

int util_sem_remove()
{
	int semid ;
	semid = util_getsem(S_IRUSR|S_IWUSR);
	if( semid == -1 )
		return -1;
	if ( semctl(semid, 0 , IPC_RMID) == -1 )
	{
		util_err_log("semctl() error!", __FILE__,__LINE__,errno);
		return -1;
	}	
	return 0; 
	
}
int util_shm_remove()
{
	
	int	shmid;
	shmid = util_getshm(S_IRUSR|S_IWUSR);
	if( shmid == -1 )
		return -1;
	if ( shmctl(shmid, 0 , (void *)IPC_RMID) == -1 )
	{
		util_err_log("shmctl() error!", __FILE__,__LINE__,errno);
		return -1;
	}
	return 0;	
}
*/
/*get a int value in config
section - name of section
key -   name of key
value - value of key
*/
int util_getinfo_int(char *section , char *key , int *nvalue , char * szFileName)
{
	char szvalue[100];
	memset( szvalue , 0 , sizeof( szvalue));
	if( util_getinfo_str( section , key , szvalue , szFileName) == -1 )
	{
		*nvalue = 0;
		return -1;
	}
	*nvalue = atoi( szvalue );
	if( errno == EINVAL || errno == ERANGE)
		return -1;
	return 0;
}



/*get a section in configuation file*/
int util_getinfo_str(char *section ,char *key , char *value , char * szFileName)
{
 	FILE *fd;
 
    char buf[1024];
    
    char cfile[100];
    char line[100];
    
	
	char thekey[100];
	char thesection[100];
	char thevalue[100];
	char cursection[100];
	char *env;
	memset( thekey  , 0 , sizeof( thekey ));
	memset( thesection , 0 , sizeof( thesection ));
	memset( cursection , 0 , sizeof( cursection ));
	memset( thevalue , 0 , sizeof( thevalue ));
    memset(cfile,0,sizeof(cfile));
    memset(buf, 0 , sizeof( buf ));
    memset(line , 0 , sizeof( line ));
    /*
    if( szInstallDir[ 0 ] != '/' )
	{
		fprintf(stderr,"Can not get install dir\n");
		return -1;
	}
	sprintf( cfile , "%s/etc/magic.ini" , szInstallDir);
	*/

    if(( fd = fopen(szFileName,"r")) == NULL)
    {
    	util_err_log("open file error!" , __FILE__ , __LINE__ , errno );
        return -1;
    }

	while( util_getline( fd , 1023 , line) != -1)
	{
		/*it is a section*/ 
		memset( thekey , 0 ,sizeof(thekey));
		memset( thevalue , 0 ,sizeof(thevalue));

		memset( thesection , 0 , sizeof(thesection ));
		util_trim( line );
		if( line[0] == '#' )
			continue;
		if( util_getsection( line , thesection ) == 0 )
		{  
			memset( cursection , 0 , sizeof( cursection ) );
			strcpy( cursection , thesection );
			continue;
		}
		/*it is a key*/
		else if( util_getkeyvalue( line , thekey , thevalue ) == 0 )
		{
			/*it's the same with query value*/

			if( strcasecmp( cursection , section ) == 0 && strcasecmp( thekey , key) == 0 )
			{

				strcpy( value ,  thevalue );
				fclose(fd );
				return 0;
			}			
		}
		else if ( strlen( line ) != 0 )
		{
			util_log("an unrecognise format:%s \n" , line );
			fclose(fd );
			return -1;
		}
		memset( line , 0 ,sizeof(line));	
	}
	if(debug)
		util_log("can not find the key: %s section: %s " , key , section );

	fclose(fd );
	return -1;
}

/*pick out section name */
/*return -1 for error format*/
int util_getsection(char *line , char *section)
{
	char *p1 , *p2;

	p1 = strchr( line , '[' );
	p2 = strchr( line , ']' );

	if( p1 == 0 || p2 == 0 )
	{

		return -1;
	}
	else 
	{
		memset( section , 0 , sizeof( section ));
		strncpy( section , p1 + 1 , p2 - p1 - 1);
		
		util_trim( section );
		return 0;
	}
	    
}

/*pick out section value */
/*return -1 for error format*/
int util_getkeyvalue(char *line ,char *thekey ,char *thevalue )
{
	char *p1;
	

	p1 = strchr( line , '=' );
	if( p1 == 0 )
		return -1;
	
	strncpy( thekey , line , p1 - line );

	strcpy( thevalue , p1 + 1 );
	util_trim( thekey );
	util_trim( thevalue );
	/*it's empty value*/
	if( strlen( thekey ) == 0 || strlen( thevalue ) == 0 )
		return -1;
	return 0;
}

/*pick out the key of line string(with out delete blank space) */
/*return -1 for error format*/
/*return 0 for success*/
int util_getkeyvalue_notrim(char *line ,char *thekey ,char *thevalue )
{
	char *p1;
	

	p1 = strchr( line , '=' );
	if( p1 == 0 )
		return -1;
	
	strncpy( thekey , line , p1 - line );

	strcpy( thevalue , p1 + 1 );
	/*empty value*/
	if( strlen( thekey ) == 0 || strlen( thevalue ) == 0 )
		return -1;
	return 0;
}

/*get one line in file
/*done not check file format
return 1 for end of file
*/
int util_getline( FILE *fd , int nLineLen , char *line)
{
	int c;
	int i = 0;

	memset( line , 0 , sizeof( line) );
	c = fgetc( fd );
	if( c == EOF )
		return -1;
	while( c == '\r' || c == '\n' )
	{
		c = fgetc( fd );

		if( c == EOF )
			return -1;
	}
	while( c != '\r' && c != '\n' && c != EOF && i < nLineLen )
	{
		
		line[ i++ ] = c;

		c = fgetc(fd );
	}
	line[ i ] = '\0';
	return i;
}


static const char *crlfs[] =
{	
	"\x0d\x0a",			/*	DOS/Windows style*/
	"\x0a\x0d",			/*	UNIX style*/
	"\x0d\x0d\x0a",     /*UNIX style2*/
	"\x0a"				/*Macintosh style*/
};
	
/*enforced getline function , you can specify the type of file*/
int util_getlineex( FILE *fd , int nLineLen , char *line ,int nstype)
{
	int c;
	int i = 0;	
	int nCrlfPtr = 0;
	char *crlf;
	memset( line , 0 , sizeof( line) );
	c = fgetc( fd );
	if( c == EOF )
		return -1;
	if( nstype < 0 || nstype > 3 )
	{
		util_err_log("util_getlineex nstype error!\n",__FILE__,__LINE__,errno );
		return -1;
	}
	crlf = (char *)crlfs[ nstype ];
	while(  c != EOF && i < nLineLen )
	{
		if ((char) c == crlf[nCrlfPtr])
		{
			nCrlfPtr ++;
			/*˻зֹλ*/
			if (crlf[nCrlfPtr] == 0)
			{
				nCrlfPtr = 0;
				break;
			}
		}
		else
			nCrlfPtr = 0;
		line[ i++ ] = c;
		c = fgetc(fd );
	}
	line[ i ] = '\0';
	return i;
}



/*delete control characters*/
void util_trim(char *buf )
{
	char tmp[1024];
	char *pbegin = buf;
	int nlen = strlen(buf);
	
	char *pend = buf + nlen - 1;

	memset( tmp , 0 , sizeof( tmp ));
	while( pbegin < buf + nlen && strchr( "\r\n\t " , *pbegin))
		pbegin ++;
	while( pend >= buf && strchr( "\r\n\t " , *pend))
		pend --;
	if( pend >= pbegin )
		strncpy( tmp , pbegin , pend - pbegin + 1);
	memset( buf , 0 , sizeof( buf ));
	strcpy( buf , tmp );
}

/*get last modified time of a file*/
int util_getfiletime(char *filename , char *time)
{
	
	struct stat buf;
	struct tm dc;
	if( stat(filename , &buf ) == -1 )
	{
		util_err_log("stat error!" , __FILE__ , __LINE__ , errno );
		return -1;
	}
	#ifndef MINGW
	localtime_r(&buf.st_mtime,&dc);
	#endif
	sprintf(time,"%04d%02d%02d%02d%02d%02d",dc.tm_year+1900,dc.tm_mon+1,dc.tm_mday,dc.tm_hour,dc.tm_min,dc.tm_sec);
	return 0;
}

int util_getfilesize(char *filename )
{
	struct stat buf;
	if( stat(filename , &buf ) == -1 )
	{
		util_err_log("stat error!" , __FILE__ , __LINE__ , errno );
		return -1;	
	}
	return buf.st_size;
}
/*get last modified time and size of file*/
/*return size of file*/
/*-1 for failed*/
long util_getfileinfo( char *filename , char *time )
{
	if( util_getfiletime(filename , time) == -1 )
		return -1;
	return  util_getfilesize(filename ) ;


}




void util_byte_copy(to,n,from)
register char *to;
register unsigned int n;
register char *from;
{
  for (;;) {
    if (!n) return; *to++ = *from++; --n;
    if (!n) return; *to++ = *from++; --n;
    if (!n) return; *to++ = *from++; --n;
    if (!n) return; *to++ = *from++; --n;
  }
}





/*test main program*/
/*
main()
{

	util_log("%s%s%d\n","test","test",3);
	util_err_log("testerr",__FILE__ , __LINE__,errno );

	util_sem_init();
	if( util_sem_lock( 1000 ) == -1)
		printf("error to lock!\n");
	
	if( util_sem_lock( 3000 ) == -1)
		printf("error lock it's right!\n");
	util_sem_unlock();
	util_sem_unlock();
	util_sem_unlock();
	if( util_sem_lock( 1000 ) == -1)
		printf("error to lock!\n");
	if( util_sem_lock( 3000 ) == -1)
		printf("error lock it's right!\n");
	util_sem_unlock();	
	util_sem_remove();	
	
	
	char time[14];
	int nsize = -1;
	memset( time , 0 , sizeof( time ));
	if( (nsize = util_getfileinfo( "/usr/ken/tsh_sun/tools.o" ,
		time ) )== -1)
		printf("getfileinfo failed!\n");
	printf( "%s , %d " ,time , nsize );
	
	
	char value[10];
	int nvalue;
	memset( value , 0 , sizeof( value ));
	
	if( util_getinfo_int( "a" , "test" , &nvalue ) == -1 )
		printf("error!\n");
	else
		printf("value=%d\n", nvalue);
	
	return 0;
	
}
*/

/*print a string with control character*/
void util_checkline(char * line , int n )
{
 	char dupline[102400];
 	int i ;
 	int j;
	memset( dupline , 0,sizeof(dupline));
	for( i = 0,j=0 ; i < n ; i++ )
	{
		if( line[i] == '\r' )
		{
			dupline[ j++ ] = '\\';
			dupline[ j++ ] = 'r';	
		}
		else if( line[i] == '\n' )
		{
			dupline[ j++ ] = '\\';
			dupline[ j++ ] = 'n';
		}
		else if(line[ i ]== '\0' )
		{
			dupline[ j++ ] = '\\';
			dupline[ j++ ] = '0';	
		}
                else if(line[ i ] < 32 )
                {
                        char a[2];
                        sprintf(a,"%d",line[i]);
                        dupline[ j++ ] = '\\';
                        dupline[ j ++ ] =a[0];
                        dupline[ j ++ ] = a[1];
                }			
		else
			dupline[ j ++] = line[i];
	}
	dupline[ j ] = '\0';
	util_log("%s\n",dupline);
}
#ifndef MINGW
/*setup alarm*/
int util_alarm(int msec)
{
	int z;
	int usec = msec * 1000;
	struct itimerval old_timer;
	struct itimerval new_timer;
	new_timer.it_interval.tv_sec = 0 ;
	new_timer.it_interval.tv_usec = 0 ;
	new_timer.it_value.tv_sec = (int)(usec / 1000000) ;
	new_timer.it_value.tv_usec = usec % 1000000 ;
	
	z = setitimer( ITIMER_REAL , &new_timer , &old_timer );
	if( z )
	{

		return -1;
	}
	if( debug )
	util_log("set timeval success:tv_sec=%d,tv_usec=%d\n",
		new_timer.it_value.tv_sec,new_timer.it_value.tv_usec );

	return 0;
}
#endif
int util_strupr(char *str)
{
	int i = 0;
	while( str[ i ] != '\0' )
	{
		str[ i ] = toupper( str[ i ] );	
		i++;
	}
	return 0;
}

int util_strlwr(char *str)
{
	int i = 0;
	while( str[ i ] != '\0' )
	{
		str[ i ] = tolower( str[ i ] );	
		i++;
	}
	
	return 0;
}
 

int util_getfilestyle(FILE *fd)
{
	int nstyle = 0 ;
	int I;
	int nSize;
	char buf[10240];
	memset( buf , 0 , sizeof(buf));
	if( ( nSize = fread( buf , 1 , sizeof(buf) , fd ) ) < 0 )
	{
		util_err_log("fread" , __FILE__,__LINE__,errno );
		return -1;
		
	}
	for (I = 0; I < nSize ; I ++)
	{
		if (buf[I] == '\n')
			break;
	}
	if (I == nSize)
	{
		/*	By default (or in the case of empty file), set DOS style*/
		nstyle = CRLF_STYLE_DOS;
	}
	else
	{
		/*	Otherwise, analyse the first occurance of line-feed character*/
		if (I > 0 && buf[I - 1] == '\r')   
		{
			if( I > 1 && buf[ I - 2 ] == '\r' )  /*"\r\r\n"*/
				nstyle = CRLF_STYLE_UNIX2;
			else
				nstyle = CRLF_STYLE_DOS;
		}
		else
		{
			if (I < nSize - 1 && buf[I + 1] == '\r' )
				nstyle = CRLF_STYLE_UNIX;
			else
				nstyle = CRLF_STYLE_MAC;				
		}
	}
	/*move file point*/
	fseek( fd , 0 , SEEK_SET );
 	return nstyle;
}

int util_isOneOf(const char c,char *s)	
{
	if(strchr (s, c) != NULL )
		return 1; 
	else
		return 0;
}


void util_replace(char *s , char src ,char to)
{
	int i = 0;
	while(s[i] != '\0')
	{
		if(s[i] == src)
			s[i] = to;
		i++;
	}
}
